######################################################################
##
## Copyright (C) 2009, Mads Chr. Olesen
##
## Author:        Mads Chr. Olesen <mads@mchro.dk>
## Description:   Filter-decorator for InstructionModel
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################

from Config import *
from dissy import InstructionModel
from Instruction import Instruction
import re

class InstructionModelHighlighter:
    def __init__(self):
        pass

    def highlight(self, row, curInstruction):
        """Possibly modify the columns of row"""
        pass

class SearchwordHighlighter(InstructionModelHighlighter):
    """Highlights anything that matches the regular expression given"""
    def __init__(self, searchPattern=None):
        InstructionModelHighlighter.__init__(self)
        self.setSearchPattern(searchPattern)

    def setSearchPattern(self, searchPattern):
        if searchPattern:
            #Match anything not in a tag (> occurs after match, but before a <)
            self.searchPattern = re.compile(searchPattern + "(?![^<]*>)")
        else:
            self.searchPattern = None

    def markup(self, pattern, string, color):
        s = ""
        last = 0
        for i in pattern.finditer(string):
            span = i.span()
            s = s + string[last:span[0]] + '<span foreground="' + color + '">' + string[span[0] : span[1] ] + '</span>'
            last = span[1]
        else:
            s = s + string[last:]
        return s

    def highlight(self, row, curInstruction):
        insn = row[InstructionModel.COLUMN_INSTRUCTION]
        strRep = row[InstructionModel.COLUMN_STR_REPRESENTATION]
        if self.searchPattern:
            strRep = self.markup(self.searchPattern, strRep, config.markupFgColor)
        row[InstructionModel.COLUMN_STR_REPRESENTATION] = strRep

class ConditionFlagHighlighter(InstructionModelHighlighter):
    """Highlights the instruction that may have set the processor condition
    flags."""

    def __init__(self):
        InstructionModelHighlighter.__init__(self)
        
    def highlight(self, row, curInstruction):
        if not curInstruction:
            return
        insn = row[InstructionModel.COLUMN_INSTRUCTION]
        arch = curInstruction.getFunction().getFile().getArch()
        #import pdb; pdb.set_trace()

        if not arch.isConditionFlagSetter(insn.getOpcode()) or \
            not arch.isConditionFlagUser(curInstruction.getOpcode()):
            return
        if curInstruction == insn:
            return
        inthezone = False
        for i in curInstruction.getFunction().getAll():
            if not isinstance(i, Instruction):
                continue
            if inthezone and arch.isConditionFlagSetter(i.getOpcode()):
                return
            if i == insn:
                inthezone = True
            if inthezone and i == curInstruction:
                row[InstructionModel.COLUMN_ADDR] = '<span foreground="green">' + \
                    row[InstructionModel.COLUMN_ADDR] + \
                    '</span>'



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Instruction class
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
from dissy.Entity import Entity, AddressableEntity

class Instruction(AddressableEntity):
    def __init__(self, function, address, encoding, insn, args):
        AddressableEntity.__init__(self, address = address, endAddress = address, baseAddress = function.baseAddress)
        self.function = function
        self.encoding = encoding
        self.insn = insn
        self.args = args
        self.outLinkAddress = None
        self.outLink = None
        arch = self.function.getFile().getArch()

        if arch.isJump(insn):
            val = arch.getJumpDestination(insn, args)
            if val != None:
                self.addLinkOut( val + self.baseAddress )

    def getFunction(self):
        return self.function

    def getOpcode(self):
        return self.insn

    def getArgs(self):
        if self.args != None:
            return self.args
        return ""
    def getOutLinkAddress(self):
        return self.outLinkAddress
    def getOutLink(self):
        return self.outLink
    def getInLinks(self):
        return self.inLink

    def hasLink(self):
        return self.outLinkAddress != None

    def addLinkIn(self, insn):
        pass # Implement this if needed
        #self.linksIn.append(insn)
    def addLinkOut(self, obj):
        self.outLinkAddress = obj

    def linkTo(self, other):
        self.outLink = other

    def link(self):
        if not self.hasLink():
            raise Exception
        other = self.function.lookup(self.getOutLinkAddress())
        if other:
            self.linkTo(other)
            other.addLinkIn(self)
        else:
            # External reference
            other = self.function.getFile().lookup(self.getOutLinkAddress())
            self.linkTo(other)
            if other == None:
                return False
            self.label = other.getLabel()
        return True

    def __str__(self):
        out = ("0x%08x " % self.address) + " " + str(self.insn)
        if self.args != None:
            out += str(" " * (20-len(out))) + str(self.args)
        return out



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   File dialogue
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import pygtk, dissy

pygtk.require('2.0')
import gtk, gobject

from dissy.Config import *
from dissy.File import File
import dissy.FunctionModel

class FileDialogue:
    def __init__(self, controller):
        file_open_dialog = gtk.FileChooserDialog(title="Open object file",
            action=gtk.FILE_CHOOSER_ACTION_OPEN,
                buttons=(gtk.STOCK_CANCEL,
                        gtk.RESPONSE_CANCEL,
                        gtk.STOCK_OPEN,
                        gtk.RESPONSE_OK))
        filter = gtk.FileFilter()
        filter.set_name("Object files")
        filter.add_pattern("*.o")
        filter.add_mime_type("application/x-object")
        filter.add_mime_type("application/x-executable")
        file_open_dialog.add_filter(filter)
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        file_open_dialog.add_filter(filter)

        if file_open_dialog.run() == gtk.RESPONSE_OK:
            filename = file_open_dialog.get_filename()
            controller.loadFile(filename)
            file_open_dialog.destroy()
        else:
            file_open_dialog.destroy()
            return



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Andrew May <acmay [at] acmay [dot] homeip [dot] net>
## Description:   PPC arch specific stuff
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import sys, architecture
from dissy.architecture import Architecture

ppc_jumps = [
              'beq',
              'bne',
              'bge',
              'b',
              'bl',
              'beq-',
              'bne-',
              'bge-',
              'blt-',
              'ble-',
              'bdnz-',
              'beq+',
              'bne+',
              'bge+',
              'blt+',
              'bdnz+',
              'ble+',
              'b-',
              ]

ppc_calls = ['bl']


class PpcArchitecture(Architecture):
    def __init__(self):
        Architecture.__init__(self, ppc_jumps, ppc_calls)

    def getJumpDestination(self, insn, args):
        r = args.split(",")
        if len(r) == 1:
            return Architecture.getJumpDestination(self, insn, args)
        return Architecture.getJumpDestination(self, insn, r[-1])



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Display the functions
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import gtk, gobject

class InfoModel:
    """ The model class holds the information we want to display """

    def __init__(self, fileContainer):
        """ Sets up and populates our gtk.TreeStore """

        self.tree_store = gtk.TreeStore( gobject.TYPE_STRING,
                                         gobject.TYPE_LONG,
                                         gobject.TYPE_STRING,
                                         gobject.TYPE_PYOBJECT
                                         )
        # Create the TreeStore
        for item in fileContainer.getFunctions():
            # Insert functions
            item.iter = self.tree_store.append(None, ("0x%08x" % item.getAddress(),
                                                      item.getSize(),
                                                      item.getLabel(),
                                                      item
                                                      ) )

    def getModel(self):
        """ Returns the model """
        if self.tree_store:
            return self.tree_store
        else:
            return None



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Configuration storage
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import cPickle, os, os.path

PROGRAM_NAME="Dissy"
PROGRAM_VERSION="9"
PROGRAM_URL="http://dissy.googlecode.com"

class Config:
    def __init__(self):
        self.configfile = os.path.expanduser("~/.dissy/config.dump")

        self.defaults = {}

        self.defaults["markupFgColor"] = "red"
        self.defaults["insnFgColor"] = "blue"
        self.defaults["highLevelCodeFgColor"] = "grey50"
        self.defaults["showHighLevelCode"] = True
        self.defaults["showInstructionInformationBox"] = True
        self.defaults["objdump"] = "objdump"
        self.defaults["readelf"] = "readelf"
        self.defaults["nm"] = "nm"
        self.defaults["version"] = PROGRAM_VERSION

        self.restoreAllDefaults()

    def restoreAllDefaults(self):
        self.markupFgColor = self.getDefault("markupFgColor")
        self.insnFgColor = self.getDefault("insnFgColor")
        self.highLevelCodeFgColor = self.getDefault("highLevelCodeFgColor")
        self.showHighLevelCode = self.getDefault("showHighLevelCode")
        self.showInstructionInformationBox = self.getDefault("showInstructionInformationBox")
        self.objdump = self.getDefault("objdump")
        self.readelf = self.getDefault("readelf")
        self.nm = self.getDefault("nm")

    def copy(self, other):
        self.markupFgColor = other.markupFgColor
        self.insnFgColor = other.insnFgColor
        self.highLevelCodeFgColor = other.highLevelCodeFgColor
        self.showHighLevelCode = other.showHighLevelCode
        self.showInstructionInformationBox = getattr(other, "showInstructionInformationBox", True)
        self.objdump = other.objdump
        self.readelf = other.readelf
        self.nm = other.nm

    def getFromEnvironment(self):
        od = os.getenv("OBJDUMP")

        if od != None:
            self.objdump = od

    def getDefault(self, which):
        return self.defaults[which]

    def load(self):
        try:
            f = open(self.configfile)
            other = cPickle.load(f)
            f.close()
            self.copy(other)
        except IOError:
            pass

    def save(self):
        try:
            os.makedirs(os.path.dirname(self.configfile), 0700)
        except OSError:
            # Already exists
            pass
        f = open(self.configfile, "w")
        cPickle.dump(self, f)
        f.close()

config = Config()

# Load the old configuration
config.load()
config.getFromEnvironment()



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Entity (address, size etc.)
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
class Entity:
    def __init__(self):
        pass


class AddressableEntity(Entity):
    def __init__(self, address=0, baseAddress = 0, endAddress = 0):
        self.label = ""
        self.address = address + baseAddress
        self.baseAddress = baseAddress
        if endAddress == 0:
            self.endAddress = self.address
        else:
            self.endAddress = endAddress + baseAddress

    def getAddress(self):
        return self.address

    def getLabel(self):
        return self.label

    def getExtents(self):
        "Return the extents of this function"
        return (self.address - self.baseAddress, self.endAddress - self.baseAddress)

    def setSize(self, size):
        "Set the size of this entity"
        self.endAddress = self.address + size

    def getSize(self):
        return self.endAddress - self.address



######################################################################
##
## Copyright (C) 2007,  Simon Kagstrom
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   History implementation
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
class History:
    def __init__(self):
        self.history = []
        self.index = 0
        self.enabled = True

    def add(self, entry):
        "Add an entry to the history"
        # Don't add twice
        if not self.enabled or self.history != [] and self.history[self.index-1] == entry:
            return None
        if self.index == 1 and len(self.history) > 1:
            self.history = []
            self.index = 0

        self.history = self.history[ : self.index] + [entry]
        self.index = self.index + 1
        return entry

    def enable(self):
        self.enabled = True

    def disable(self):
        self.enabled = False

    def back(self):
        "Go back one step"
        if self.index-1 <= 0:
            raise Exception("End of history")
        self.index = self.index - 1
        return self.history[self.index-1]

    def forward(self):
        "Go forward one step"
        if self.index >= len(self.history):
            raise Exception("End of history")
        self.index = self.index + 1
        return self.history[self.index-1]

    def current(self):
        "Get the current entry"
        return self.history[self.index]

    def isFirst(self):
        "Is this the first entry?"
        return self.index <= 0

    def isLast(self):
        "Is this the last entry?"
        return self.index >= len(self.history)

    def getAllEntries(self):
        "Return everything in the current history"
        return self.history

if __name__ == "__main__":
    h = History()
    print "Adding some entries"
    h.add(1)
    h.add(2)
    h.add(3)
    print h.getAllEntries()
    print "Back one step:", h.back()
    h.add(4)
    print "Add, all entries:", h.getAllEntries()
    print "Back two, forward"
    print h.back()
    print h.back()
    try:
        # Should raise
        print h.back()
    except:
        print "Got expected exception (back)"
    print h.forward()
    print h.forward()
    try:
        print h.forward()
    except:
        print "Got expected exception (fwd)"
    print "All entries", h.getAllEntries()
    h.back()
    h.back()
    h.add(5)
    print h.getAllEntries()
    h.add(6)
    print h.getAllEntries()



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   MIPS arch specific stuff
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import dissy.architecture, re
from dissy.architecture import Architecture

REGISTER_REGEXP = "(?:[tsakv]{1}[0-9]{1})|sp|ra|fp|gp|at"
ADDRESS_REGEXP  = "[0-9,a-f,A-F]+"

mips_jumps = ['bgez',
              'bnez',
              'beqz',
              'blez',
              'bgez',
              'bltz',
              'bgtz',
              'bc1f',
              'bc1t',
              'beq',
              'bne',
              'b',
              'jal',
              'j',
              ]

mips_calls = ['jal']

instr_descriptions = {
    'lbu': 'Load Unsigned Byte',
    'lb': 'Load signed Byte',
    'li': 'Load Immediate value to destination register',
    'lui' : 'Load Upper Immediate (set destination to v << 16)',
    'move' : 'Copy between registers',
    'jal' : 'Jump And Link, jump to destination and store return address in ra (r31)',
    'or' : 'Bitwise or',
    'sb' : 'Store Byte to memory',
    'sw' : 'Store 32-bit word to memory',
    'sh' : 'Store 16-bit word to memory',
    'lb' : 'Load Byte from memory, signed',
    'lw' : 'Load 32-bit value from memory',
    'lh' : 'Load 16-bit value from memory, signed',
    'lbu' : 'Load 16-bit value from memory, unsigned',
    'lhu' : 'Load 16-bit value from memory, unsigned',
}


class MipsArchitecture(Architecture):
    def __init__(self):
        Architecture.__init__(self, mips_jumps, mips_calls)
        self.jumpRegexp = re.compile("(?:(" + REGISTER_REGEXP + "),)+" + "(" + ADDRESS_REGEXP + ")");

    def getJumpDestination(self, insn, args):
        r = self.jumpRegexp.match(args)
        if r == None:
            return Architecture.getJumpDestination(self, insn, args)
        return Architecture.getJumpDestination(self, insn, r.group(2))

    def getInstructionInfo(self, instruction):
        opcode = instruction.getOpcode()
        args = str(instruction.getArgs())
        description = instr_descriptions.get(instruction.getOpcode(), '')

        return {'shortinfo': opcode + " " + args,
                'description': description,
                }



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Describes one file
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import cgi, os, sys

sys.path.append(".")
import dissy, dissy.architecture

from dissy.Function import *
from dissy.Instruction import *
from dissy.Data import *
from dissy.Entity import AddressableEntity

ADDRESS_REGEXP  = "[0-9,a-f,A-F]+"
FUNCTION_REGEXP = "(?:[.]*)[_,0-9,a-z,A-Z,\:,\*,\,\(,\), ,<,>,~,\.]+"

symbolRegexp = re.compile("(" + ADDRESS_REGEXP + ")* *(" + ADDRESS_REGEXP + ")* ([A,B,C,D,G,I,N,R,S,T,U,V,W,a,b,c,d,g,i,n,r,s,t,u,v,w,-,?]{1}) ("+ FUNCTION_REGEXP + "){1}")

# Followed by size, but let's just skip it
linuxKernelCrashRegexp = re.compile("(" + FUNCTION_REGEXP + "){1}" + "\+[0x]*(" + ADDRESS_REGEXP + "){1}")

TYPE_UNDEFINED = 0
TYPE_TEXT = 1
TYPE_RODATA = 2
TYPE_DATA = 3
TYPE_BSS  = 4

typeToClass = {
    TYPE_TEXT : Function,
    TYPE_RODATA : RoData,
    TYPE_DATA : Data,
    TYPE_BSS : Bss,
}

def getObjType(s):
    if s in ('u', 'U'):
        return TYPE_UNDEFINED
    elif s in ('d', 'D'):
        return TYPE_DATA
    elif s in ('r', 'R'):
        return TYPE_RODATA
    elif s in ('b', 'B', 's', 'S', 'c', 'C'):
        return TYPE_BSS
    elif s in ('t', 'T'):
        return TYPE_TEXT
    return TYPE_DATA


class File(AddressableEntity):
    def __init__(self, filename=None, baseAddress = 0):
        AddressableEntity.__init__(self, baseAddress = baseAddress)
        self.symbols = []
        self.functions = []
        self.data = []
        self.filename = filename
        self.arch = "intel"
        if filename != None:
            self.arch = dissy.architecture.getArchitecture(self.getArchStr())
            if self.hasSymbols():
                self.parse()
            else:
                self.parseNoSymbols()

    def getArchStr(self):
        "Get the architecture of the file"
        arch = "intel" # Assume Intel architecture
        f = os.popen("%s -h --wide %s" % (config.readelf, self.filename))
        for line in f:
            words = line.split()
            if len(words) >= 2 and words[0] == "Machine:":
                arch = words[1].lower()
                break
        f.close()
        return arch

    def lookup_int(self, address):
        for sym in self.symbols:
            extents = sym.getExtents()
            if address >= extents[0] and address < extents[1]:
                return sym
        return None

    def lookup_str(self, label):
        for sym in self.symbols:
            if label == sym.getLabel():
                return sym
        return None

    def lookup(self, param):
        "Lookup a label or an address"
        if isinstance(param, long):
            return self.lookup_int(param)
        return self.lookup_str(param)

    def getArch(self):
        return self.arch

    def hasSymbols(self):
        return True

    def link(self):
        for fn in self.functions:
            unresolved = fn.link()
            for insn in unresolved:
                other = self.lookup(insn.getAddress())
                if other:
                    insn.linkTo(other)

    def parse(self):
        "Parse the functions from this file (with symbols)"

        f = os.popen("%s --numeric-sort --demangle %s" % (config.nm, self.filename))
        lines = f.readlines()
        f.close()

        for line in lines:
            r = symbolRegexp.match(line)
            if r == None:
                continue
            address = 0
            size = 0
            if r.group(1) != None:
                address = long("0x" + r.group(1), 16)
            if r.group(2) != None:
                size = long("0x" + r.group(2), 16)
            objType = getObjType(r.group(3))
            label = cgi.escape(r.group(4))

            if size == 0:
                idx = lines.index(line)
                if idx < len(lines)-1:
                    # The size is next line.adr - this line.adr
                    s = symbolRegexp.match(lines[idx+1])
                    if s == None or s and s.group(1) == None:
                        # Nope, doesn't work...
                        continue

                    nextAdr = long("0x" + s.group(1), 16)
                    size = nextAdr - address
                else:
                    # FIXME: This is a bug - the last symbol will be too small.
                    # This can be fixed by e.g., using the section size
                    size = 0

            if objType == TYPE_UNDEFINED:
                continue

            if objType == TYPE_TEXT:
                sym = typeToClass[objType](self, address, label, size)
                self.functions.append(sym)
                self.symbols.append(sym)
#            else:
#                sym = typeToClass[objType](self, address, label, size)
#                self.data.append(sym)


    def parseNoSymbols(self):
        "Parse the functions from this file (without symbols)"

        callDests = []
        f = os.popen("%s --disassemble --demangle %s" % (config.objdump, self.filename))
        for line in f:
            r = insnRegExp.match(line)
            if r != None:
                if self.arch.isCall(r.group(3)):
                    dst = self.arch.getJumpDestination(r.group(3), r.group(4))
                    if dst != None:
                        callDests.append(dst)
        # Sort the calls
        callDests.sort()
        count = 0
        for call in callDests[:-1]:
            next = callDests[callDests.index(call)+1]
            if next-call == 0:
                # Empty
                continue
            fn = Function(self, call, "func%d" % (count), next-call)
            # This has not been prepared yet...
            fn.stream = None
            self.functions.append(fn)
            count = count + 1
        f.close()

    def getFunctions(self):
        return self.functions

    def getData(self):
        return self.functions

    def __str__(self):
        out = "%s: " % (self.filename)
        for fn in self.functions:
            out = out + str(fn)
        return out


if __name__ == "__main__":
    if len(sys.argv) < 2:
        print "Arg: filename (ELF)"
        sys.exit(1)
    f = File(sys.argv[1])
    f.parse()



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Intel arch specific stuff
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import sys, architecture
from dissy.architecture import Architecture

intel_jumps = ['jmp',
               'call'
               ]
intel_calls = ['call']

intel_conditionflag_setters = ['cmp', 'cmpb', 'cmps', 'cmpw', 'cmpl', 'cmpq', 'test']

intel_conditionflag_users = ['']

intel_instr_descriptions = {
    'push': 'Push Word onto Stack',
    'pushl': 'Push Long onto Stack',
    'mov': 'Move',
    'movl': 'Move Long',
    'cmp': 'Compare',
    'cmpb': 'Compare Byte',
    'lea': 'Load Effective Address',
    'add': 'Add',
    'jmp': 'Unconditional Jump',
    'pop': 'Pop Word off Stack',
    'ret': 'Return from Procedure',
    'sub': 'Subtract',
    'xor': 'eXclusive Or',
    'and': 'Logical And',
    'nop': 'NoOp',
    'call': 'Procedure Call',
    'hlt': 'Halt',
    'test': """Test for Bit Pattern
Performs a logical AND of the two operands, updating the condition flags without saving the results""",
    'leave': 'Restore stack for procedure exit',
    'xchg': 'Exchange',
    'sar': 'Shift Arithmetic Right',
    'sal': 'Shift Arithmetic Left',
    'shr': 'Shift Logical Right',
    'shl': 'Shift Logical Left',
}

intel_lists_inited = False
if not intel_lists_inited:
    conditional_instructions = {
        'j': "Jump if %s",
    }
    conditions = {
        'a': 'Above',
        'ae': 'Above or Equal',
        'b': 'Below',
        'be': 'Below or Equal',
        'c': 'Carry set',
        'cxz': 'CX Zero',
        'e': 'Equal',
        'g': 'Greater (Signed)',
        'ge': 'Greater or Equal (Signed)',
        'l': 'Less (Signed)',
        'le': 'Less or Equal (Signed)',
        'na': 'Not Above',
        'nae': 'Not Above or Equal',
        'nb': 'Not Below',
        'nbe': 'Not Below or Equal',
        'nc': 'Not Carry',
        'ne': 'Not Equal',
        'ng': 'Not Greater (Signed)',
        'nge': 'Not Greater or Equal (Signed)',
        'nl': 'Not Less (Signed)',
        'nle': 'Not Less or Equal (Signed)',
        'no': 'Not Overflow (Signed)',
        'np': 'No Parity',
        'ns': 'Not Signed (Signed)',
        'nz': 'Not Zero',
        'o': 'Overflow (Signed)',
        'p': 'Parity',
        'pe': 'Parity Even',
        'po': 'Parity Odd',
        's': 'Signed (Signed)',
        'z': 'Zero',

    }
    for i in ['j']:
        for c in conditions:
            intel_instr_descriptions[i + c] = conditional_instructions[i] % (conditions[c])
            intel_conditionflag_users += [i + c]
            intel_jumps += [i + c]


class IntelArchitecture(architecture.Architecture):
    def __init__(self):
        architecture.Architecture.__init__(self, intel_jumps, intel_calls,
            intel_conditionflag_setters, intel_conditionflag_users)

    def getInstructionInfo(self, instruction):
        opcode = instruction.getOpcode()
        args = str(instruction.getArgs())
        description = intel_instr_descriptions.get(instruction.getOpcode(), '')

        return {'shortinfo': opcode + " " + args,
                'description': description,
                }



######################################################################
##
## Copyright (C) 2006,  Blekinge Institute of Technology
##
## Author:        Simon Kagstrom <simon.kagstrom@gmail.com>
## Description:   Preferences dialog
##
## Licensed under the terms of GNU General Public License version 2
## (or later, at your option). See COPYING file distributed with Dissy
## for full text of the license.
##
######################################################################
import pygtk

# General warning: The code in this file is pure horror! Go somewhere else
# if you suffer from heart problems.

pygtk.require('2.0')
import gtk, gobject

from Config import *

def colorToString(color):
    return "#%04x%04x%04x" % (color.red, color.green, color.blue)

class PreferencesDialogue:
    def __init__(self, main_program):
        self.main_program = main_program

        dialog = gtk.Dialog("%s - Preferences" % (PROGRAM_NAME))
        defaults = gtk.Button("Defaults")
        cancel = gtk.Button("Cancel", gtk.STOCK_CANCEL)
        ok = gtk.Button("OK", gtk.STOCK_OK)

        table = gtk.Table(3, 3, False)
        objdump = gtk.Entry()
        readelf = gtk.Entry()
        nm = gtk.Entry()
        insnColor = gtk.ColorButton(gtk.gdk.color_parse(config.insnFgColor))
        markupColor = gtk.ColorButton(gtk.gdk.color_parse(config.markupFgColor))
        highLevelColor = gtk.ColorButton(gtk.gdk.color_parse(config.highLevelCodeFgColor))

        showHighLevel = gtk.CheckButton("Show high level code")
        showInstructionInfo = gtk.CheckButton("Show instruction information")

        cancel.connect("clicked", lambda w: dialog.destroy())
        ok.connect("clicked", self.okSelected,
                   dialog, objdump, readelf, nm, showHighLevel,
                   showInstructionInfo, insnColor, markupColor, highLevelColor)
        defaults.connect("clicked", self.defaultsSelected,
                         objdump, readelf, nm, showHighLevel, showInstructionInfo,
                         insnColor, markupColor, highLevelColor)

        objdump.set_text(config.objdump)
        readelf.set_text(config.readelf)
        nm.set_text(config.nm)

        showHighLevel.set_active(config.showHighLevelCode)
        showInstructionInfo.set_active(config.showInstructionInformationBox)

        table.attach(gtk.Label("Objdump:"), 0, 1, 0, 1,  ypadding=2)
        table.attach(gtk.Label("Readelf:"), 0, 1, 1, 2,  ypadding=2)
        table.attach(gtk.Label("nm:"), 0, 1, 2, 3,  ypadding=2)

        table.attach(gtk.Label("Instruction color:"), 0, 1, 3, 4,  ypadding=2)
        table.attach(gtk.Label("Highlight color:"), 0, 1, 4, 5,  ypadding=2)
        table.attach(gtk.Label("High-level code color:"), 0, 1, 5, 6,  ypadding=2)

        table.attach(objdump, 1, 2, 0, 1,  ypadding=2)
        table.attach(readelf, 1, 2, 1, 2,  ypadding=2)
        table.attach(nm, 1, 2, 2, 3,  ypadding=2)

        table.attach(insnColor, 1, 2, 3, 4,  ypadding=2)
        table.attach(markupColor, 1, 2, 4, 5,  ypadding=2)
        table.attach(highLevelColor, 1, 2, 5, 6,  ypadding=2)

        table.attach(showHighLevel, 0, 2, 6, 7,  ypadding=6)
        table.attach(showInstructionInfo, 0, 2, 7, 8,  ypadding=6)

        dialog.vbox.pack_start(table, True, True, 0)
        dialog.action_area.pack_start(defaults, True, True, 0)
        dialog.action_area.pack_start(cancel, True, True, 0)
        dialog.action_area.pack_start(ok, True, True, 0)
        dialog.show_all()

    def defaultsSelected(self, widget, objdump, readelf, nm, showHighLevel, showInstructionInfo, insnColor, markupColor, highLevelColor):
        config.restoreAllDefaults()
        objdump.set_text(config.objdump)
        readelf.set_text(config.readelf)
        nm.set_text(config.nm)

        insnColor.set_color( gtk.gdk.color_parse(config.getDefault("insnFgColor")) )
        markupColor.set_color( gtk.gdk.color_parse(config.getDefault("markupFgColor")) )
        highLevelColor.set_color( gtk.gdk.color_parse(config.getDefault("highLevelCodeFgColor")) )

        showHighLevel.set_active(config.showHighLevelCode)
        showInstructionInfo.set_active(config.showInstructionInfo)

    def okSelected(self, widget, dialog, objdump, readelf, nm,
                   showHighLevelCode, showInstructionInfo, insnColor, markupColor, highLevelColor):
        if objdump.get_text() == "":
            config.objdump = config.getDefault("objdump")
        else:
            config.objdump = objdump.get_text()

        if readelf.get_text() == "":
            config.readelf = config.getDefault("readelf")
        else:
            config.readelf = readelf.get_text()

        if nm.get_text() == "":
            config.nm = config.getDefault("nm")
        else:
            config.nm = nm.get_text()

        config.insnFgColor = colorToString(insnColor.get_color())
        config.markupFgColor = colorToString(markupColor.get_color())
        config.highLevelCodeFgColor = colorToString(highLevelColor.get_color())

        config.showHighLevelCode = showHighLevelCode.get_active()
        config.showInstructionInformationBox = showInstructionInfo.get_active()
        config.save()

        dialog.destroy()
        self.main_program.setInformationBox()

